ReforgeSummary = LibStub("AceAddon-3.0"):NewAddon("ReforgeSummary", "AceEvent-3.0", "AceHook-3.0")
local core = ReforgeSummary
local ReforgingInfo = LibStub("LibReforgingInfo-1.0")
local Gratuity = LibStub("LibGratuity-3.0")

local displayOrder = {
    INVSLOT_HEAD,
    INVSLOT_NECK,
    INVSLOT_SHOULDER,
    INVSLOT_BACK,
    INVSLOT_CHEST,
    INVSLOT_WRIST,

    INVSLOT_HAND,
    INVSLOT_WAIST,
    INVSLOT_LEGS,
    INVSLOT_FEET,
    INVSLOT_FINGER1,
    INVSLOT_FINGER2,
    INVSLOT_TRINKET1,
    INVSLOT_TRINKET2,

    INVSLOT_MAINHAND,
    INVSLOT_OFFHAND,
    INVSLOT_RANGED,
}
-- Mapped from reforging stat ids. Not using LibReforgingInfo because "rating"
-- is appended on hit and crit
local extraShort = {
	ITEM_MOD_SPIRIT_SHORT,
	STAT_DODGE,
	STAT_PARRY,
    HIT,
    CRIT_ABBR,
	STAT_HASTE,
	STAT_EXPERTISE,
	STAT_MASTERY,
}
    
local anchor = CreateFrame("Frame") -- Anchor for first item in display list
local buttons = {} -- maps inventory slot to frame

function core:OnEnable()
    if IsAddOnLoaded("Blizzard_ReforgingUI") then
        self:Setup()
    else
        self:RegisterEvent("ADDON_LOADED", function(_, name)
            if name == "Blizzard_ReforgingUI" then
                self:Setup()
                self:UnregisterEvent("ADDON_LOADED")
            end
        end)
    end
end

function core:Setup()
    -- Prevent item pickup when clicking after reforge
    self:SecureHook("ReforgingFrame_AddItemClick", function() ClearCursor() end)

    -- Right click on frame while in reforger will back out
    self:HookScript(ReforgingFrame, "OnMouseUp", function(self, button) if button == "RightButton" then ReforgingFrame_AddItemClick() end end)

    -- Check when to show
    self:SecureHook("ReforgingFrame_Update", "CheckUpdate")

    -- Update on inventory change
    self:RegisterEvent("UNIT_INVENTORY_CHANGED", "CheckUpdate")

    -- Run Update
    self:CheckUpdate()
end

function core:CheckUpdate()
    if ReforgingFrame.missingDescription:IsVisible() then
        -- Hide gray text
        ReforgingFrame.missingDescription:SetText("")

        -- Check for init
        if not next(buttons) then
            self:CreateButtons()
        end

        -- Show all items
        self:Buttons("Update")
    else
        -- Hide all items
        self:Buttons("Hide")
    end
end

function core:Buttons(func, ...)
    for _, frame in pairs(buttons) do
        frame[func](frame, ...)
    end
end

function core:CreateButtons()
    -- Position anchor
    anchor:ClearAllPoints()
    anchor:SetPoint("TOPLEFT", ReforgingFrameHorzBar, "TOPLEFT", 0, 15)
    anchor:SetSize(1, 1)

    local last = anchor
    local width = 380
    local font = GameFontNormal

    for _,slot in ipairs(displayOrder) do
        local frame = CreateFrame("Button")
        frame:SetParent(ReforgingFrame)
        frame:SetSize(width, 14)
        frame:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -1)
        frame:RegisterForClicks("LeftButtonUp")
        
        -- Item icon
        local icon = CreateFrame("Frame")
        icon:SetParent(frame)
        icon:SetSize(14, 14)
        icon:SetPoint("LEFT")
        local iconTexture = icon:CreateTexture()
        iconTexture:SetAllPoints()

        -- Item name
        local name = frame:CreateFontString()
        name:SetFontObject(font)
        name:SetSize(170, 14)
        name:SetJustifyH("LEFT")
        name:SetPoint("LEFT", icon, "RIGHT", 3, 0)

        -- Stat amount
        local refAmt = frame:CreateFontString()
        refAmt:SetFontObject(font)
        refAmt:SetSize(45, 14)
        refAmt:SetJustifyH("RIGHT")
        refAmt:SetPoint("LEFT", icon, "RIGHT", 175, 0)

        -- Stat label from
        local fromStat = frame:CreateFontString()
        fromStat:SetFontObject(font)
        fromStat:SetSize(65, 14)
        fromStat:SetJustifyH("LEFT")
        fromStat:SetPoint("LEFT", icon, "RIGHT", 225, 0)

        -- Stat label to
        local toStat = frame:CreateFontString()
        toStat:SetFontObject(font)
        toStat:SetSize(65, 14)
        toStat:SetJustifyH("LEFT")
        toStat:SetPoint("LEFT", icon, "RIGHT", 300, 0)

        -- Handlers
        frame:SetScript("OnClick", function(self)
            PickupInventoryItem(slot)
            SetReforgeFromCursorItem()
        end)
        frame:SetScript("OnEnter", function(self)
            GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT", width, -16)
            GameTooltip:SetInventoryItem("player", slot)
            GameTooltip:Show()

            name:SetTextColor(1, 1, 1, 1)
            refAmt:SetTextColor(1, 1, 1, 1)
            fromStat:SetTextColor(1, 1, 1, 1)
            toStat:SetTextColor(1, 1, 1, 1)
        end)
        frame:SetScript("OnLeave", function(self)
            GameTooltip:Hide()

            name:SetTextColor(font:GetTextColor())
            refAmt:SetTextColor(font:GetTextColor())
            fromStat:SetTextColor(font:GetTextColor())
            toStat:SetTextColor(font:GetTextColor())
        end)

        -- Set info for this button
        frame.Update = function(self)
            local link = GetInventoryItemLink("player", slot)
            if not link then
                self:Hide()
                return
            end

            self:Show()
            -- Update name and icon
            name:SetText(link:match("%[(.+)%]"))
            iconTexture:SetTexture(GetInventoryItemTexture("player", slot))

            -- Get stats
            local stats = ReforgingInfo:GetReforgeID(link)
            if not stats then
                -- Not reforged, clear it
                refAmt:SetText("")
                fromStat:SetText("")
                toStat:SetText("")
            else
                -- Set stat labels
                local from, to = ReforgingInfo:GetReforgedStatIDs(stats)
                fromStat:SetText(extraShort[from])
                toStat:SetText(extraShort[to])

                -- Scan tooltip for amount
                Gratuity:SetInventoryItem("player", slot)
                local _, target = ReforgingInfo:GetReforgedStatNames(stats)
                -- Using MultiFind because we don't know case sensitivity yet
                local left, right = "%s.-(%%d+)", "(%%d+).-%s"
                local _, _, amt = Gratuity:MultiFind(2, nil, nil, true, left:format(target), right:format(target), left:format(target:lower()), right:format(target:lower()))
                if not amt then
                    error("Could not find", target, "in", name:GetText())
                    return
                end
                refAmt:SetText(amt)
            end
        end

        last = frame
        buttons[slot] = frame
    end
end
